{-# OPTIONS_GHC -Wunused-imports #-}

-- | Backend for generating highlighted, hyperlinked HTML from Agda sources.

module Agda.Interaction.Highlighting.HTML
  ( htmlBackend
  ) where

import Agda.Interaction.Highlighting.HTML.Backend as Exports
  ( htmlBackend
  )
