{-# OPTIONS_GHC -Wunused-imports #-}

-- | Generating highlighted and aligned LaTeX from literate Agda source.

module Agda.Interaction.Highlighting.LaTeX
  ( module Exports
  ) where

import Agda.Interaction.Highlighting.LaTeX.Backend as Exports
  ( latexBackend
  )
